/*-
 * Copyright (c) 1993, 1994, 1995 Michael B. Durian.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Michael B. Durian.
 * 4. The name of the the Author may be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include "MidiDev.h"

MidiDevice::MidiDevice() : name(0), error(0), play_callback(0),
    record_callback(0), repeat(0)
{
}

MidiDevice::MidiDevice(const char *dev) : error(0), play_callback(0),
    record_callback(0), repeat(0)
{

	name = new char [strlen(dev) + 1];
	assert(name != 0);
	strcpy(name, dev);
}

MidiDevice::MidiDevice(const MidiDevice &md)
{

	delete name;
	name = new char[strlen(md.name) + 1];
	assert(name != 0);
	strcpy(name, md.name);
	delete error;
	error = 0;
	play_callback = md.play_callback;
	record_callback = md.record_callback;
	repeat = md.repeat;
}

MidiDevice::~MidiDevice()
{

	delete name;
	delete error;
}

void
MidiDevice::SetName(const char *dev)
{

	delete name;
	name = new char[strlen(dev) + 1];
	assert(name != 0);
	strcpy(name, dev);
}

void
MidiDevice::SetError(const char *err)
{

	delete error;
	error = new char[strlen(err) + 1];
	assert(error != 0);
	strcpy(error, err);
}
